/*
  SugarTrace
  Copyright by Naoyuki Tamura (tamura@kobe-u.ac.jp)
 */
function calc(c, expr) {
    var value = 0;
    var s = (expr + "").split(" ");
    for (var i = 0; i < s.length; i++) {
	var str = s[i];
	var factor = 1;
	if (str.match(/^-/)) {
	    str = str.substring(1);
	    factor = -1;
	}
	if (str.match(/^([\.\d]+)/)) {
	    factor *= RegExp.$1;
	    str = str.replace(/^([\.\d]+)/, "");
	}
	var v = 1;
	if (c.d[str] != undefined) {
	    v = c.d[str];
	} else if (str.match(/^(\w+)\.(\w+)$/)) {
	    var data = c.shapes[RegExp.$1]["data"];
	    v = data[RegExp.$2];
	}
	v = factor * v;
	value += v;
    }
    return value;
}

function arrowHead(x1, y1, x, y, h, a) {
    var b = Math.PI * a / 180;
    var path;
    var t = Math.atan2(y1-y, x1-x);
    var dx1 = h * Math.cos(t - b);
    var dy1 = h * Math.sin(t - b);
    var dx2 = h * Math.cos(t + b);
    var dy2 = h * Math.sin(t + b);
    return ["M", x+dx1, y+dy1, "L", x, y, x+dx2, y+dy2];
}

function calcPath(c, p) {
    var path = [];
    var arrows = [];
    var ar = [];
    var d = [];
    for (var i = 0; i < p.length; i++) {
	if (p[i].match(/^:?[<>]/)) {
	    ar[d.length] = p[i];
	} else if (p[i].match(/^:/)) {
	    if (p[i].match(/^:S(-?[\.\d]+)$/)) {
		path.push("S");
		var a = Math.tan(RegExp.$1 * Math.PI / 180);
		var x1 = d[d.length-2];
		var y1 = d[d.length-1];
		var x2 = calc(c, p[i+1]);
		var y2 = calc(c, p[i+2]);
		var t = Math.atan2(y2-y1, x2-x1);
		var r = Math.sqrt(Math.pow(x2-x1,2)+Math.pow(y2-y1,2)) / 2;
		var x = x1 + r * (Math.cos(t) - a * Math.sin(t));
		var y = y1 + r * (Math.sin(t) + a * Math.cos(t));
		d.push(x); path.push(x);
		d.push(y); path.push(y);
	    } else {
		path.push(p[i].substring(1));
	    }
	} else {
	    var xy = calc(c, p[i]);
	    d.push(xy); path.push(xy);
	}
    }
    for (var i = 0; i < ar.length; i++) {
	if (ar[i] && ar[i].match(/^:?([<>])([\.\d]*)(,([\.\d]+))?$/)) {
	    var h = RegExp.$2 || c.arrowSize || 15;
	    var a = RegExp.$4 || c.arrowAngle || 15;
	    var arrow;
	    if (RegExp.$1 == ">") {
		arrow = arrowHead(d[i-4], d[i-3], d[i-2], d[i-1], h, a);
	    } else {
		arrow = arrowHead(d[i+2], d[i+3], d[i], d[i+1], h, a);
	    }
	    arrows.push(arrow);
	}
    }
    return { path: path, arrows: arrows };
}

function calcArgs(c, s, args) {
    var data = {};
    for (var i = 0; i < args.length; i++) {
	if (s[args[i]] != undefined) {
	    data[args[i]] = calc(c, s[args[i]]);
	}
    }
    if (s.type == "circle") {
	data.w = data.h = data.r;
    }
    if (data.w && data.h) {
	data.L = data.x - data.w/2;
	data.C = data.x;
	data.R = data.x + data.w/2;
	data.T = data.y - data.h/2;
	data.M = data.y;
	data.B = data.y + data.h/2;
    }
    return data;
}

function draw(canvasName, c) {
    var canvasNode = document.getElementById(canvasName);
    canvasNode.innerHTML = "";
    var raphael = Raphael(canvasNode, c.width, c.height);
    // raphael.rect(0, 0, c.width-1, c.height-1);
    c.colSize = (c.width - c.margin[1] - c.margin[3]) / c.cols;
    c.rowSize = (c.height - c.margin[0] - c.margin[2]) / c.rows;
    c.d = {};
    c.d.C = c.colSize;
    c.d.R = c.rowSize;
    c.d.c = c.colSize - c.colsep;
    c.d.r = c.rowSize - c.rowsep;
    c.d.w = c.cols * c.colSize;
    c.d.W = c.width;
    c.d.h = c.rows * c.rowSize;
    c.d.H = c.height;
    for (var i = 0; i <= c.cols; i++) {
	c.d["C" + i] = c.margin[3] + c.colSize * i;
	c.d["c" + i] = c.d["C" + i] + c.colSize / 2;
    }
    for (var j = 0; j <= c.rows; j++) {
	c.d["R" + j] = c.margin[0] + c.rowSize * j;
	c.d["r" + j] = c.d["R" + j] + c.rowSize / 2;
    }
    for (var i = 0; i < c.shapes.length; i++) {
	var s = c.shapes[i];
	var d = {};
	if (s.type == "rect") {
	    s.data = calcArgs(c, s, ["x", "y", "w", "h", "r"]);
	    var x = s.data.x-s.data.w/2;
	    var y = s.data.y-s.data.h/2;
	    s.element = raphael.rect(x, y, s.data.w, s.data.h, s.data.r);
	    s.element.attr("stroke-width", c.strokeWidth || 1);
	} else if (s.type == "circle") {
	    s.data = calcArgs(c, s, ["x", "y", "r"]);
	    s.element = raphael.circle(s.data.x, s.data.y, s.data.r/2);
	    s.element.attr("stroke-width", c.strokeWidth || 1);
	} else if (s.type == "ellipse") {
	    s.data = calcArgs(c, s, ["x", "y", "w", "h"]);
	    s.element = raphael.ellipse(s.data.x, s.data.y, s.data.w/2, s.data.h/2);
	    s.element.attr("stroke-width", c.strokeWidth || 1);
	} else if (s.type == "path") {
	    s.data = calcArgs(c, s, ["h"]);
	    var p = calcPath(c, s.p);
	    if (p.arrows) {
		var set = raphael.set();
		var path = raphael.path(p.path);
		set.push(path);
		s.elements = [ path ];
		for (var j = 0; j < p.arrows.length; j++) {
		    var head = raphael.path(p.arrows[j]);
		    head.attr("fill", "#000");
		    set.push(head);
		    s.elements.push(head);
		}
		s.element = set;
	    } else {
		s.element = raphael.path(p.path);
	    }
	    s.element.attr("stroke-width", c.strokeWidth || 1);
	} else if (s.type == "fill") {
	    var p = calcPath(c, s.p);
	    p.path.push("Z");
	    s.element = raphael.path(p.path);
	    s.element.attr("stroke-width", c.strokeWidth || 1);
	} else if (s.type == "text") {
	    s.data = calcArgs(c, s, ["x", "y"]);
	    s.element = raphael.text(s.data.x, s.data.y, s.t);
	    var size = s.f || c.fontSize || "20px";
	    s.element.attr("font-size", size);
	}
	if (s.attr) {
	    s.element.attr(s.attr);
	}
	if (s.name) {
	    c.shapes[s.name] = s;
	}
    }
}

function setAttr(c, name, attr) {
    c.shapes[name].element.attr(attr);
}
