/*
  SugarTrace
  Copyright by Naoyuki Tamura (tamura@kobe-u.ac.jp)
 */
var url0 = "http://localhost";
var pathQuit = "/trace/quit";
var pathRestart = "/trace/restart";
var pathStepForward = "/trace/step";
var variables = new Array();
var variableMap = new Array();
var decisions = new Array();
var learnts = new Array();
var done = false;
var wait = false;
var mode = "";
var varsColumn = 5;
var messageNode;
var varsNode;
var decisionsNode;
var learntsNode;
var consoleNode;
var drawDiagram;

function init() {
    variables = new Array();
    variableMap = new Array();
    decisions = new Array();
    learnts = new Array();
    done = false;
    wait = false;
    mode = "";
    messageNode = document.getElementById("message");
    messageNode.innerHTML = "";
    varsNode = document.getElementById("vars");
    varsNode.innerHTML = "";
    decisionsNode = document.getElementById("decisions");
    decisionsNode.innerHTML = "";
    learntsNode = document.getElementById("learnts");
    learntsNode.innerHTML = "";
    consoleNode = document.getElementById("console");
    consoleNode.innerHTML = "";
    if (drawDiagram) {
	document.getElementById("canvas").innerHTML = "";
    }
}

function incColButton() {
    varsColumn++;
    showVariables();
    return true;
}

function decColButton() {
    if (varsColumn > 1) {
	varsColumn--;
	showVariables();
    }
    return true;
}

function hideShow(element) {
    var node = document.getElementById(element);
    if (node.style.display == "none") {
	node.style.display = "block";
    } else {
	node.style.display = "none";
    }
    return true;
}

function hideShowVarNames() {
    var nodes = document.getElementsByClassName("var_name");
    for (var i = 0; nodes.length; i++) {
	if (nodes[i].style.display == "none") {
	    nodes[i].style.display = "block";
	} else {
	    nodes[i].style.display = "none";
	}
    }
    return true;
}

function incFontSize(element) {
    var node = document.getElementById(element);
    var size = 100;
    if (node.style.fontSize.match(/^([0-9\.]+)%$/)) {
	size = RegExp.$1;
    }
    size *= 1.10;
    node.style.fontSize = size + "%";
    return true;
}

function decFontSize(element) {
    var node = document.getElementById(element);
    var size = 100;
    if (node.style.fontSize.match(/^([0-9\.]+)%$/)) {
	size = RegExp.$1;
    }
    size /= 1.10;
    node.style.fontSize = size + "%";
    return true;
}

function quitButton() {
    if (! confirm("Really Quit?")) {
	return false;
    }
    sendRequest(pathQuit, null);
    return true;
}

function restartButton() {
    sendRequest(pathRestart, callback);
    init();
    return true;
}

function stepButton() {
    if (done) {
	alert("Already finished");
	return;
    }
    if (wait) {
	alert("Waiting for the next response...");
	return false;
    }
    mode = "step";
    getNext();
    return true;
}

function skipButton() {
    if (done) {
	alert("Already finished");
	return;
    }
    if (wait) {
	alert("Waiting for the next response...");
	return false;
    }
    mode = "skip";
    showMessage("wait...");
    getNext();
    return true;
}

function runButton() {
    if (done) {
	alert("Already finished");
	return;
    }
    if (wait) {
	alert("Waiting for the next response...");
	return false;
    }
    mode = "run";
    showMessage("wait...");
    getNext();
    return true;
}

function pauseButton() {
    mode = "";
    return true;
}

function getNext() {
    if (wait) {
	setTimeout("getNext()", 50);
	return;
    }
    wait = true;
    sendRequest(pathStepForward, callback);
}

function callback(http) {
    if (http.readyState != 4) {
	return;
    }
    done = false;
    wait = false;
    if (http.status != 200) {
	alert("Server is not responding : " + http.status);
	return;
    }
    var result = http.responseText;
    if (result != "") {
	result = eval("(" + result + ")");
    }
    var pause = false;
    var cmd = result[0];
    var args = result.slice(1);
    var msg = result.join(" ");
    if (cmd == "startSolver") {
	pause = true;
	if (drawDiagram) {
	    drawDiagram();
	}
    } else if (cmd == "stopSolver") {
	done = true;
	pause = true;
    } else if (cmd == "restartSolver") {
	pause = true;
    } else if (cmd == "sat" || cmd == "unsat") {
	done = true;
	pause = true;
    } else if (cmd == "found") {
	pause = true;
	showVariables();
	clearModified();
    } else if (cmd == "decide") {
	// decide, lit
	pause = true;
	decisions.push(args[0]);
	showDecisions();
	showVariables();
	clearModified();
    } else if (cmd == "conflict") {
	// conflict, lit, ...
	pause = true;
	showVariables();
	clearModified();
    } else if (cmd == "backjump") {
	// backjump, level
	pause = true;
	var level = args[0];
	decisions = decisions.slice(0, level);
	showDecisions();
	clearModified();
    } else if (cmd == "learnt") {
	// learnt, lit, ...
	pause = true;
	if (args.length > 8) {
	    msg = result.slice(0, 8).join(" ") + " ...";
	}
	learnts.push(args.join(" "));
	showLearnts();
	clearModified();
    } else if (cmd == "propagate") {
	// propagate, lit
	if (args.length > 8) {
	    msg = result.slice(0, 8).join(" ") + " ...";
	}
	showVariables();
    } else if (cmd == "undo") {
	// undo, lit
	if (args.length > 8) {
	    msg = result.slice(0, 8).join(" ") + " ...";
	}
	showVariables();
    } else if (cmd == "var") {
	// var, int|bool, name, newdomain, olddomain
	msg = "";
	updateVariable(args[0], args[1], args[2], args[3]);
	getNext();
	return;
    } else if (cmd == "vars") {
	// vars, array of var
	msg = "";
	for (var i in args[0]) {
	    var v = args[0][i];
	    updateVariable(v[0], v[1], v[2], v[3]);
	}
	getNext();
	return;
    }
    if (msg) {
	showConsole(msg);
	if (mode == "skip" && ! pause) {
	    msg = "skipping : " + msg;
	} else if (mode == "run" && ! done) {
	    msg = "running : " + msg;
	}
	showMessage(msg);
    }
    if (mode == "skip" && ! pause) {
	getNext();
    } else if (mode == "run" && ! done) {
	getNext();
    }
}

function Variable(type, name, domain) {
    this.type = type;
    this.name = name;
    this.domain = domain || "";
    this.modifiedNow = true;
    this.modified = true;
}

function updateVariable(type, name, domain, oldDomain) {
    var variable = variableMap[name];
    if (variable == null) {
	variable = new Variable(type, name, domain);
	variables.push(variable);
	variableMap[name] = variable;
    } else {
	variable.domain = domain;
	if (oldDomain && domain != oldDomain) {
	    variable.modified = true;
	    variable.modifiedNow = true;
	}
    }
}

function showVariables() {
    var n = variables.length;
    var s = "";
    for (var i = 0; i < n; i += varsColumn) {
	s += "<tr class='var'>";
	for (var j = i; j < i + varsColumn && j < n; j++) {
	    var variable = variables[j];
	    var domain_style = "domain";
	    if (variable.modifiedNow) {
		domain_style += " modifiedNow";
		variable.modifiedNow = false;
		if (drawDiagram) {
		    drawDiagram(variable);
		}
	    } else if (variable.modified) {
		domain_style += " modified";
	    }
	    s += "<td class='var_name'>" + variable.name + "</td>";
            s += "<td></td>";
	    s += "<td class='" + domain_style + "'>" + variable.domain + "</td>";
	}
	s += "<td class='var_name'></td>";
	s += "</tr>";
    }
    varsNode.innerHTML = s;
}

function clearModified() {
    for (var i = 0; i < variables.length; i++) {
	variables[i].modified = false;
	variables[i].modifiedNow = false;
    }
}

function showDecisions() {
    var s = "";
    if (decisions) {
	for (var i = 0; i < decisions.length; i++) {
	    s += "<li>" + decisions[i] + "</li>";
	}
    }
    decisionsNode.innerHTML = s;
}

function showLearnts() {
    var s = "";
    if (learnts) {
	for (var i = 0; i < learnts.length; i++) {
	    s += "<li>" + learnts[i] + "</li>";
	}
    }
    learntsNode.innerHTML = s;
}

function showMessage(str) {
    messageNode.innerHTML = str;
}

function showConsole(str) {
    consoleNode.innerHTML += str + "\n";
}

function sendRequest(path, callback) {
    var port = window.location.port;
    var url = url0 + ":" + port + path;
    var http = createHttpRequest();
    http.open("GET", url, true);
    if (callback) {
	http.onreadystatechange = function () { callback(http); };
    }
    http.send(null);
}

function createHttpRequest() {
    if (window.ActiveXObject) {
	try {
	    return new ActiveXObject("Msxml2.XMLHTTP");
	} catch (e) {
	}
	try {
	    return new ActiveXObject("Microsoft.XMLHTTP");
	} catch (e) {
	}
    } else if (window.XMLHttpRequest) {
	return new XMLHttpRequest();
    }
    return null;
}
