package sugartracer;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.util.concurrent.LinkedBlockingDeque;

import org.json.JSONArray;

import sugartracer.decoder.Decoder;
import sugartracer.decoder.Variable;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

public class ServerTracer extends Tracer {
    public int port = 8080;
    public String htmlDir = null;
    public String htmlPath = "/html";
	public String commandPath = "/trace";
	public HttpServer server = null;
    private boolean quit = false;
    private boolean done = false;
    private int queueSize = 100000;
	private LinkedBlockingDeque<JSONArray> queue = null;
	
	public ServerTracer(SatSolver solver, Decoder decoder, String htmlDir, int port) {
	    super(solver, decoder);
        this.htmlDir = htmlDir;
		this.port = port;
	}
	
    public class HtmlHandler implements HttpHandler {
        int buffSize = 1024;
        File dir;

        public HtmlHandler(File dir) {
            this.dir = dir;
        }

        public synchronized void handle(HttpExchange exchange) throws IOException {
            if (quit)
                return;
            // Headers headers = exchange.getRequestHeaders();
            String path0 = exchange.getHttpContext().getPath();
            String path1 = exchange.getRequestURI().getPath();
            String path = htmlDir + path1.substring(path0.length());
            OutputStream out = exchange.getResponseBody();
            if (debug)
                System.out.println("GET " + path);
            File file = new File(path);
            if (file.isFile()) {
                exchange.sendResponseHeaders(200, 0);
                InputStream in = new FileInputStream(file);
                byte[] buff = new byte[buffSize];
                while (in.available() > 0) {
                    int length = in.read(buff);
                    out.write(buff, 0, length);
                }
            } else {
                int code = 404;
                String msg = code + " Not Found " + exchange.getRequestURI();
                exchange.sendResponseHeaders(code, 0);
                out.write(msg.getBytes());
            }
            out.close();
        }
    }
    
    public class CommandHandler implements HttpHandler {
        public synchronized void handle(HttpExchange exchange) throws IOException {
            if (quit)
                return;
            String path0 = exchange.getHttpContext().getPath();
            String path1 = exchange.getRequestURI().getPath();
            String path = path1.substring(path0.length());
            // String query = exchange.getRequestURI().getQuery();
            OutputStream out = exchange.getResponseBody();
            String str = null;
            if (debug)
                System.out.println("GET " + path);
            try {
                if (path.equals("/quit")) {
                    stopTracer();
                    str = "";
                } else if (path.equals("/restart")) {
                    restartTracer();
                    str = "";
                } else if (path.equals("/step")) {
                    str = stepTracer();
                }
            } catch (TracerException e) {
            }
            int code = 200;
            if (str == null) {
                code = 404;
                str = code + " Not Found " + exchange.getRequestURI();
            }
            exchange.sendResponseHeaders(code, 0);
            out.write(str.getBytes());
            out.close();
            if (debug)
                System.out.println(str);
        }
    }
    
    public synchronized void startTracer() throws TracerException {
        try {
            server = HttpServer.create(new InetSocketAddress(port), 0);
            File dir = new File(htmlDir);
            if (dir.isDirectory()) {
                server.createContext(htmlPath, new HtmlHandler(dir));
            }
            server.createContext(commandPath, new CommandHandler());
            server.start();
            System.out.println("SugarTrace server starting on port " + port);
            quit = false;
            done = false;
            queue = new LinkedBlockingDeque<JSONArray>(queueSize);
            super.startTracer();
            while (! quit) {
                try {
                    wait();
                } catch (InterruptedException e) {
                }
            }
            server.stop(0);
            System.out.println("SugarTrace server stopped");
            done = true;
        } catch (IOException e) {
            throw new TracerException(e.getMessage());
        }
    }

    @Override
    public synchronized void stopTracer() throws TracerException {
        System.out.println("Stopping SugarTrace server");
        super.stopTracer();
        quit = true;
        notifyAll();
        while (! done) {
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }
    }
    
    public synchronized void restartTracer() throws TracerException {
        System.out.println("Restarting SugarTrace server");
        done = true;
        super.stopTracer();
        quit = false;
        done = false;
        queue = new LinkedBlockingDeque<JSONArray>(queueSize);
        System.out.println("SugarTrace server restarting on port " + port);
        super.startTracer();
        notifyAll();
    }
    
	public synchronized String stepTracer() throws TracerException {
        JSONArray json = null;
        while (true) {
            json = queue.poll();
	        if (json != null)
	            break;
            try {
                wait();
            } catch (InterruptedException e) {
            }
	    }
        notifyAll();
        return json.toString();
	}

	private synchronized void showJSON(JSONArray json) {
	    while (! done && ! queue.offer(json)) {
            try {
                wait();
            } catch (InterruptedException e) {
            }
	    }
        notifyAll();
	}
	
    @Override
    public synchronized void show(String[] strs) {
        JSONArray json = new JSONArray();
        for (String s : strs) {
            json.put(s);
        }
        showJSON(json);
    }

    @Override
    public synchronized void showAssignment() {
        JSONArray json1 = new JSONArray();
        for (Variable v : decoder.getVariables()) {
            if ((showAuxVars || ! v.isAux()) && v.isModified(decoder.getAssignment())) {
                JSONArray json2 = new JSONArray();
                for (String s : v.toStrings(decoder.getAssignment(), true))
                    json2.put(s);
                json1.put(json2);
            }
        }
        if (json1.length() > 0) {
            JSONArray json = new JSONArray();
            json.put("vars");
            json.put(json1);
            showJSON(json);
        }
    }
}
