package sugartracer;

import sugartracer.decoder.Decoder;
import sugartracer.sat4j.Sat4j;

/**
 * SugarTracer
 * @author Naoyuki Tamura
 */
public class SugarTracer {
    private static boolean quick = false;
    private static boolean debug = false;
    
    public static void runTextTracer(String cnfFileName, String mapFileName) throws TracerException {
        SatSolver solver = new Sat4j(cnfFileName); 
        Tracer tracer = new TextTracer(solver, new Decoder(mapFileName));
        solver.setTracer(tracer);
        tracer.quick = quick;
        tracer.debug = debug;
        tracer.startTracer();
        tracer.stopTracer();
    }
    
    public static void runServerTracer(String cnfFileName, String mapFileName, String htmlDir, int port) throws TracerException {
        SatSolver solver = new Sat4j(cnfFileName); 
        Tracer tracer = new ServerTracer(solver, new Decoder(mapFileName), htmlDir, port);
        solver.setTracer(tracer);
        tracer.quick = quick;
        tracer.debug = debug;
        tracer.startTracer();
        tracer.stopTracer();
    }
    
    public static void main(String[] args) {
        String htmlDir = null;
        int port = 8080;
        int i = 0;
        while (i < args.length) {
            if (args[i].equals("-server") && i + 1 < args.length) {
                i++;
                htmlDir = args[i]; 
            } else if (args[i].equals("-port") && i + 1 < args.length) {
                i++;
                port = Integer.parseInt(args[i]); 
            } else if (args[i].equals("-quick")) {
                quick = true;
            } else if (args[i].equals("-debug")) {
                debug = true;
            } else {
                break;
            }
            i++;
        }
        if (i + 2 != args.length) {
            System.out.println("Usage: java sugartracer.SugarTracer [options] cnfFileName mapFileName");
            System.out.println("\t-quick : Faster response by ommiting 'propgate' and 'undo' information");
            System.out.println("\t-server dir : HTTP server mode");
            System.out.println("\t-port num (default 8080) : Port number of HTTP server mode");
            System.out.println("\t-debug");
            System.exit(1);
        }
        String cnfFileName = args[i];
        String mapFileName = args[i + 1];
        try {
            if (htmlDir == null) {
                runTextTracer(cnfFileName, mapFileName);
            } else {
                runServerTracer(cnfFileName, mapFileName, htmlDir, port);
            }
        } catch (TracerException e) {
            e.printStackTrace();
        }
    }
}
