package sugartracer.sat4j;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.sat4j.core.LiteralsUtils;
import org.sat4j.minisat.SolverFactory;
import org.sat4j.reader.DimacsReader;
import org.sat4j.reader.ParseFormatException;
import org.sat4j.reader.Reader;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IConstr;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.ISolverService;
import org.sat4j.specs.Lbool;
import org.sat4j.specs.SearchListener;
import org.sat4j.specs.TimeoutException;

import sugartracer.SatSolver;
import sugartracer.TracerException;

public class Sat4j extends SatSolver implements SearchListener<ISolverService> {
    private static final long serialVersionUID = 5428504901632558767L;

    private ISolver solver;
    
    public Sat4j(String cnfFileName) {
        super(cnfFileName);
    }

    @Override
    public void startSolver() throws TracerException {
        solver = SolverFactory.newDefault();
        solver.setSearchListener(this);
        Reader reader = new DimacsReader(solver);
        try {
            IProblem problem = reader.parseInstance(cnfFileName);
            problem.findModel();
        } catch (ContradictionException e) {
            tracer.end(false);
        } catch (FileNotFoundException e) {
            throw new TracerException(e.getMessage());
        } catch (ParseFormatException e) {
            throw new TracerException(e.getMessage());
        } catch (IOException e) {
            throw new TracerException(e.getMessage());
        } catch (TimeoutException e) {
            throw new TracerException(e.getMessage());
        }
    }

    @Override
    public void stopSolver() throws TracerException {
        solver.setSearchListener(null);
        solver.reset();
    }

    @Override
    public void restartSolver() throws TracerException {
        stopSolver();
        startSolver();
    }

    public void assuming(int lit) {
        tracer.decide(lit);
    }

    public void propagating(int lit, IConstr reason) {
        tracer.propagate(lit);
    }

    public void backtracking(int lit) {
        tracer.undo(lit);
    }

    public void backjump(int backjumpLevel) {
        tracer.backjump(backjumpLevel);
    }

    public void conflictFound(IConstr confl, int dlevel, int trailLevel) {
        int[] clause = new int[confl.size()];
        for (int i = 0; i < confl.size(); i++)
            clause[i] = LiteralsUtils.toDimacs(confl.get(i));
        tracer.conflict(clause);
    }

    public void conflictFound(int lit) {
        tracer.conflict(new int[] { lit });
    }

    public void learn(IConstr clause) {
        int[] clause1 = new int[clause.size()];
        for (int i = 0; i < clause.size(); i++)
            clause1[i] = LiteralsUtils.toDimacs(clause.get(i));
        tracer.learnt(clause1);
    }

    public void solutionFound(int[] model) {
        tracer.found();
    }

    public void adding(int lit) {
    }

    public void delete(int[] clause) {
    }

    public void init(ISolverService solverService) {
    }

    public void restarting() {
        tracer.restart();
    }

    public void cleaning() {
    }

    public void beginLoop() {
    }
    
    public void start() {
    }

    public void end(Lbool result) {
        if (result.equals(Lbool.TRUE))
            tracer.end(true);
        else if (result.equals(Lbool.FALSE))
            tracer.end(false);
    }
}
